package de.jalin.ldapadmin.ldap;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.apache.commons.codec.binary.Base64;


public class PasswordValidator {

    private static final int MIN_PASSWORD_LEN = 8;
	private static final Base64 BASE64 = new Base64();
	private static final String LABEL = "{SSHA512}";
	private static final String SALT_CHARACTERS = 
			  "abcdefghijklmnopqrstuvwxyz"
			+ "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
			+ "0123456789$_";

    public void validate(final String password) throws SimplePasswordException {
        if (password == null || password.isEmpty()) {
            throw new SimplePasswordException("password required");
        }
        if (password.length() < MIN_PASSWORD_LEN) {
            throw new SimplePasswordException("minimal password length is " + MIN_PASSWORD_LEN + " characters");
        }
        int hasLowerCaseChar = 0;
        int hasUpperCaseChar = 0;
        int hasDigits = 0;
        int hasSpecialChar = 0;
        for (int idx = 0; idx < password.length(); idx++) {
            final char test = password.charAt(idx);
            final int type = Character.getType(test);
            if (type == Character.DECIMAL_DIGIT_NUMBER) {
                hasDigits = 1;
            } else {
                if (type == Character.LOWERCASE_LETTER) {
                    hasLowerCaseChar = 1;
                } else {
                    if (type == Character.UPPERCASE_LETTER) {
                        hasUpperCaseChar = 1;
                    } else {
                        hasSpecialChar = 1;
                    }
                }
            }
        }
        if (hasDigits + hasLowerCaseChar + hasUpperCaseChar + hasSpecialChar < 3) {
            throw new SimplePasswordException("a password requires 3 out of 4 "
                    + "different character types: lowercase, uppercase, digits and special characters");
        }
    }

    public static String randomSalt() {
    	final StringBuffer buffer = new StringBuffer();
    	for (int i=0; i < 8; i++) {
    		double randomValue = Math.random() * 64.0f - 0.5d;
    		int randomIndex = Math.round((float)randomValue);
    		if (randomIndex < 0 || randomIndex >= SALT_CHARACTERS.length()) {
    			randomIndex = 0;
    		}
    		buffer.append(SALT_CHARACTERS.charAt(randomIndex));
    	}
    	return buffer.toString();
    }
    
	public String createSaltedSHA512Hash(String passwd) throws NoSuchAlgorithmException {
		return createSaltedSHA512Hash(randomSalt(), passwd);
    }
    
	public static String createSaltedSHA512Hash(String salt, String passwd) throws NoSuchAlgorithmException {
		final byte[] saltBytes = salt.getBytes();
		MessageDigest sha = MessageDigest.getInstance("SHA-512");
		sha.reset();
		sha.update(passwd.getBytes());
		sha.update(saltBytes);
		byte[] pwHash = sha.digest();
		final byte[] hashBytes = new byte[pwHash.length + saltBytes.length];
		System.arraycopy(pwHash, 0, hashBytes, 0, pwHash.length);
		System.arraycopy(saltBytes, 0, hashBytes, pwHash.length, saltBytes.length);
		final String encode = BASE64.encodeAsString(hashBytes);
		return LABEL + new String(encode);
	}
 
}
