package de.jalin.ldapadmin.web;

import java.util.SortedMap;
import java.util.TreeMap;

import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpSession;

import de.jalin.ldapadmin.beans.Group;
import de.jalin.ldapadmin.beans.User;
import de.jalin.ldapadmin.ldap.LDAPConfig;
import de.jalin.ldapadmin.ldap.LDAPSession;
import de.jalin.ldapadmin.ldap.LDAPSessionException;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AbstractLDAPServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    protected static final Logger LOG = Logger.getLogger("LDAP");

    private SortedMap<String, User> users;
    private SortedMap<String, Group> groups;

    protected LDAPSession ldapSession;
    protected LDAPConfig config;

    protected void loadData() {
        users = new TreeMap<>();
        groups = new TreeMap<>();
    }

    protected User getUser(final String uid) {
        return users.get(uid);
    }

    protected Group getGroup(final String gid) {
        return groups.get(gid);
    }

    @Override
    public void init() throws ServletException {
        super.init();
        try {
            config = LDAPConfig.getConfig();
            ldapSession = new LDAPSession(config.getLdapProviderUrl(), config.getLdapSecurityPrincipal(), config.getLdapSecurityPassword());
        } catch (LDAPSessionException ex) {
            Logger.getLogger(AbstractLDAPServlet.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    @Override
    public void destroy() {
        super.destroy();
        try {
            ldapSession.close();
        } catch (NamingException e) {
            LOG.severe(e.getMessage());
        }
    }

    protected void throwServletException(final HttpSession session, final Exception e) throws ServletException {
        session.setAttribute("servletexception", e);
        LOG.severe(e.getMessage());
        throw new ServletException(e);
    }

    protected void cleanSession(final HttpSession httpSession) {
        httpSession.removeAttribute("errormessage");
        httpSession.removeAttribute("successmessage");
        httpSession.removeAttribute("servletexception");
    }
    
}
