package de.jalin.ldapadmin.web;

import java.io.IOException;
import java.util.SortedMap;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import de.jalin.ldapadmin.beans.User;
import de.jalin.ldapadmin.ldap.GroupsDAO;
import de.jalin.ldapadmin.ldap.LDAPSessionException;
import de.jalin.ldapadmin.ldap.UsersDAO;

@WebServlet(name = "LdapGroups", urlPatterns = {"/groups"}, loadOnStartup = 1)
public class GroupsServlet extends AbstractLDAPServlet {

    private static final long serialVersionUID = 1L;

    @Override
    protected void doGet(final HttpServletRequest req, final HttpServletResponse resp)
            throws ServletException, IOException {
        final HttpSession httpSession = req.getSession();
        cleanSession(httpSession);
        final UsersDAO usersDAO = new UsersDAO(ldapSession);
        final GroupsDAO groupsDAO = new GroupsDAO(ldapSession);
        try {
            final SortedMap<String, User> loadedUsers = usersDAO.loadUsers();
            httpSession.setAttribute("users", loadedUsers);
            httpSession.setAttribute("groups", groupsDAO.loadGroups(loadedUsers));
        } catch (LDAPSessionException e) {
            throwServletException(httpSession, e);
        }
        req.getRequestDispatcher("/groups.jsp").forward(req, resp);
    }

}
