package de.jalin.ldapadmin.web;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.SortedMap;

import javax.naming.directory.SearchResult;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import de.jalin.ldapadmin.beans.User;
import de.jalin.ldapadmin.beans.ValidationException;
import de.jalin.ldapadmin.ldap.GroupsDAO;
import de.jalin.ldapadmin.ldap.LDAPSessionException;
import de.jalin.ldapadmin.ldap.SimplePasswordException;
import de.jalin.ldapadmin.ldap.UsersDAO;

@WebServlet(name = "LdapProfile", urlPatterns = {"/", "/profile", "/profile/*"}, loadOnStartup = 1)
public class ProfileServlet extends AbstractLDAPServlet {

    private static final long serialVersionUID = 1L;
    
    @Override
    public void init() throws ServletException {
    	super.init();
    }
    
    @Override
    protected void doGet(final HttpServletRequest req, final HttpServletResponse resp)
            throws ServletException, IOException {
        final HttpSession httpSession = req.getSession();
        String userDN = "";
        try {
            final String remoteUID = req.getRemoteUser();
            final List<SearchResult> list = ldapSession.search("ou=users", "uid", remoteUID);
            if (list != null && list.size() > 0) {
                final SearchResult first = list.get(0);
                userDN = first.getNameInNamespace();
            }
            cleanSession(httpSession);
            String operation = req.getParameter("op");
            if (operation == null || operation.isEmpty()) {
                operation = "profile";
            }
            httpSession.setAttribute("operation", operation);
            httpSession.setAttribute("formdisabled", "profile".equals(operation) ? "disabled" : "");
            httpSession.setAttribute("iddisabled", "disabled");
            final UsersDAO usersDAO = new UsersDAO(ldapSession);
            final GroupsDAO groupsDAO = new GroupsDAO(ldapSession);
            SortedMap<String, User> users = usersDAO.loadUsers();
            httpSession.setAttribute("users", users);
            httpSession.setAttribute("groups", groupsDAO.loadGroups(users));
            httpSession.setAttribute("user", users.get(userDN));
        } catch (LDAPSessionException e) {
            throwServletException(httpSession, e);
        }
        req.getRequestDispatcher("/user.jsp").forward(req, resp);
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp)
            throws ServletException, IOException {
        final Messages messages = new Messages(req.getLocale());
        final HttpSession httpSession = req.getSession();
        cleanSession(httpSession);
        final String pathInfo = req.getPathInfo();
        String dn = ""; 
        if (pathInfo != null && pathInfo.length() > 1 && pathInfo.charAt(0) == '/') {
            dn = pathInfo.substring(1);
        }
        String operation = req.getParameter("op");
        if (operation == null || operation.isEmpty()) {
            operation = "none";
        }
        final UsersDAO usersDAO = new UsersDAO(ldapSession);
        final String password = req.getParameter("password");
        final String password2 = req.getParameter("password2");
        final User usr = (User) httpSession.getAttribute("user");
        try {
            if (password != null && !password.isEmpty()) {
                if (password2 == null || !password2.equals(password)) {
                    throw new ValidationException("password2", messages.getString("ProfileServlet.passwords_donot_match"));
                } else {
                    usr.setAndValidatePassword(password);
                }
            }
        } catch (SimplePasswordException e) {
            httpSession.setAttribute("user", usr);
            httpSession.setAttribute("errormessage", messages.getString("ResetPasswordServlet.simple_password"));
            req.getRequestDispatcher("/user.jsp").forward(req, resp);
            return;
        } catch (ValidationException e) {
            httpSession.setAttribute("user", usr);
            httpSession.setAttribute("errormessage", messages.getString("ProfileServlet.inputfield") + e.getFieldname() + " " + e.getCondition());
            req.getRequestDispatcher("/user.jsp").forward(req, resp);
            return;
        } catch (NoSuchAlgorithmException e) {
        	throwServletException(httpSession, e);		}
        try {
            if ("password".equals(operation)) {
                final User oldValue = usersDAO.read(dn);
                usr.setLogin(oldValue.getLogin());
                usr.setDn(dn);
                usersDAO.update(usr);
                httpSession.setAttribute("successmessage", messages.getString("ProfileServlet.password_changed"));
                httpSession.setAttribute("operation", "profile");
                httpSession.setAttribute("formdisabled", "disabled");
                httpSession.setAttribute("iddisabled", "disabled");
                httpSession.setAttribute("user", usr);
                req.getRequestDispatcher("/user.jsp").forward(req, resp);
            }
        } catch (LDAPSessionException e) {
            final String excMessage = e.getMessage();
            if (excMessage != null && excMessage.contains("invalid reuse of password")) {
                httpSession.setAttribute("user", usr);
                httpSession.setAttribute("errormessage", messages.getString("ProfileServlet.invalid_reuse"));
                req.getRequestDispatcher("/user.jsp").forward(req, resp);
                return;
            }
            throwServletException(httpSession, e);
        }
    }

}
