package de.jalin.ldapadmin.ldap;

import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import javax.naming.NamingException;
import javax.naming.NoPermissionException;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.jalin.ldapadmin.beans.User;

public class TestUpdateAsSimpleUser {

    private static final String USERS_DN = "uid=mic,ou=users,dc=domain,dc=example,dc=com";

    @BeforeClass
    public static void setupClass() throws Exception {
        DirectoryServiceRunner.assureServiceRunning("domain");
    }

    @Before
    public void setUp() throws Exception {
        final LDAPSession bindUserSession = new LDAPSession("ldap://localhost:10389/dc=domain,dc=example,dc=com", "uid=application,ou=bind,dc=domain,dc=example,dc=com", "app-secret");
        final UsersDAO dao = new UsersDAO(bindUserSession);
        final User newUser = new User();
        newUser.setDn(USERS_DN);
        newUser.setDisplayname("Paul Petersen");
        newUser.setEmail("paul.petersen@example.com");
        newUser.setFirstname("Paul");
        newUser.setLastname("Petersen");
        newUser.setLogin("plp");
        newUser.setMobile("0163 1234567");
        newUser.setPhone("030 12345678");
        newUser.setPassword("geheim");
        dao.create(newUser);
        bindUserSession.close();
    }

    @After
    public void tearDown() throws Exception {
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
//		Thread.sleep(10 * 60000L);
    }

    @Test
    public void test() {
        try {
            final LDAPSession simpleUserSession = new LDAPSession("ldap://localhost:10389/dc=domain,dc=example,dc=com", "uid=plp,ou=users,dc=domain,dc=example,dc=com", "geheim");
            final UsersDAO dao = new UsersDAO(simpleUserSession);
            final User existingUser = dao.loadUsers().get(USERS_DN);
            assertNull("user already exists", existingUser);
            final User newUser = new User();
            newUser.setDn(USERS_DN);
            newUser.setDisplayname("Micha Michaelsen");
            newUser.setEmail("micha.michaelsen@example.com");
            newUser.setFirstname("Michael");
            newUser.setLastname("Michaelsen");
            newUser.setLogin("mic");
            newUser.setMobile("0163 2234567");
            newUser.setPhone("030 22345678");
            newUser.setPassword("geheim");
            try {
                dao.create(newUser);
                dao.loadUsers().get(USERS_DN);
                fail("should not exist");
            } catch (LDAPSessionException e) {
                assertTrue(e.getCause() instanceof NoPermissionException);
            }
            simpleUserSession.close();
        } catch (LDAPSessionException | RequiredAttributeException | AlreadyBoundException | NamingException e) {
            fail(e.getMessage());
        }
    }

}
