package de.jalin.ldapadmin.ldap;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.jalin.ldapadmin.beans.User;

public class TestUpdateUser {

    private static final String USERS_DN = "uid=kla,ou=users,dc=domain,dc=example,dc=com";

    private LDAPSession session;

    @BeforeClass
    public static void setupClass() throws Exception {
        DirectoryServiceRunner.assureServiceRunning("domain");
    }

    @Before
    public void setUp() throws Exception {
        session = new LDAPSession("ldap://localhost:10389/dc=domain,dc=example,dc=com", "uid=admin,ou=system", "streng-geheim");
    }

    @After
    public void tearDown() throws Exception {
        session.close();
        session = null;
    }

    @Test
    public void test() {
        final UsersDAO dao = new UsersDAO(session);
        try {
            final User existingUser = dao.loadUsers().get(USERS_DN);
            assertNull("user already exists", existingUser);
            final User newUser = new User();
            newUser.setDn(USERS_DN);
            newUser.setDisplayname("Klaas Clahsen");
            newUser.setEmail("klaas.clahsen@example.com");
            newUser.setFirstname("Klaas");
            newUser.setLastname("Klahsen");
            newUser.setLogin("kla");
            newUser.setMobile("0163 2234567");
            newUser.setPhone("030 22345678");
            newUser.setPassword("geheim");
            dao.create(newUser);
            final User createdUser = dao.loadUsers().get(USERS_DN);
            assertNotNull("should exist", createdUser);
            createdUser.setLastname("Clahsen");
            createdUser.setPassword("strenggeheim");
            dao.update(createdUser);
            final User updatedUser = dao.loadUsers().get(USERS_DN);
            assertTrue("should be updated", "Clahsen".equals(updatedUser.getLastname()));
        } catch (LDAPSessionException | RequiredAttributeException | AlreadyBoundException e) {
            fail(e.getMessage());
        }
    }

}
